@@ -414,6 +414,12 @@ handle_dev_fd_read(int fd, void *data)
 					  PORT_BUFSIZE);
     }
 
+    if (port->dev_monitor != NULL) {
+	controller_write(port->dev_monitor,
+			 (char *) port->dev_to_tcp_buf,
+			 port->dev_to_tcp_buf_count);
+    }
+
     if (port->dev_to_tcp_buf_count < 0) {
 	/* Got an error on the read, shut down the port. */
 	syslog(LOG_ERR, "dev read error for port %s: %m", port->portname);
@@ -443,19 +449,15 @@ handle_dev_fd_read(int fd, void *data)
 	}
     }
 
+ retry_write:
     write_count = write(port->tcpfd,
 			port->dev_to_tcp_buf,
 			port->dev_to_tcp_buf_count);
 
-    if (port->dev_monitor != NULL) {
-	controller_write(port->dev_monitor,
-			 port->dev_to_tcp_buf,
-			 port->dev_to_tcp_buf_count);
-    }
     if (write_count == -1) {
 	if (errno == EINTR) {
-	    /* EINTR means we were interrupted, just retry by returning. */
-	    return;
+	    /* EINTR means we were interrupted, just retry. */
+	    goto retry_write;
 	}
 
 	if (errno == EAGAIN) {
@@ -589,18 +591,20 @@ handle_tcp_fd_read(int fd, void *data)
 	}
     }
 
+    if (port->tcp_monitor != NULL) {
+	controller_write(port->tcp_monitor,
+			 (char *) port->tcp_to_dev_buf,
+			 port->tcp_to_dev_buf_count);
+    }
+
+ retry_write:
     write_count = write(port->devfd,
 			port->tcp_to_dev_buf,
 			port->tcp_to_dev_buf_count);
-    if (port->tcp_monitor != NULL) {
-	controller_write(port->tcp_monitor,
-			 port->tcp_to_dev_buf,
-			 port->tcp_to_dev_buf_count);
-    }
     if (write_count == -1) {
 	if (errno == EINTR) {
-	    /* EINTR means we were interrupted, just retry by returning. */
-	    return;
+	    /* EINTR means we were interrupted, just retry. */
+	    goto retry_write;
 	}
 
 	if (errno == EAGAIN) {
@@ -836,6 +840,16 @@ display_banner(port_info_t *port)
 		    add_port_tcp_char(port, *t);
 		break;
 
+	    case 's':
+		/* ser2net serial parms. */
+		{
+		    char str[15];
+		    serparm_to_str(str, sizeof(str), &(port->termctl));
+		    for (t=str; *t; t++)
+			add_port_tcp_char(port, *t);
+		}
+		break;
+
 	    case '0': case '1': case '2': case '3': case '4': case '5':
 	    case '6': case '7':
 		/* Octal digit */
@@ -973,7 +987,8 @@ setup_tcp_port(port_info_t *port)
     }
 
     /* Turn off BREAK. */
-    if (ioctl(port->devfd, TIOCCBRK) == -1) {
+    if (port->enabled != PORT_RAWLP &&
+              ioctl(port->devfd, TIOCCBRK) == -1) {
 	close(port->tcpfd);
 	close(port->devfd);
 	syslog(LOG_ERR, "Could not turn off break for device %s port %s: %m",
@@ -2066,7 +2081,7 @@ static void
 com_port_handler(void *cb_data, unsigned char *option, int len)
 {
     port_info_t *port = cb_data;
-    char outopt[16];
+    unsigned char outopt[16];
     struct termios termio;
     int val;
     
@@ -2077,7 +2092,7 @@ com_port_handler(void *cb_data, unsigned char *option, int len)
     case 0: /* SIGNATURE? */
 	outopt[0] = 44;
 	outopt[1] = 0;
-	strcpy(outopt+2, "ser2net");
+	strcpy((char *) outopt+2, "ser2net");
 	telnet_send_option(&port->tn_data, outopt, 9);
 	break;
 
@@ -2183,18 +2198,18 @@ com_port_handler(void *cb_data, unsigned char *option, int len)
 
 	val = 0;
 	if (tcgetattr(port->devfd, &termio) != -1) {
-	    /* We don't support 1.5 stop bits. */
-	    if ((option[2] == 1) || (option[2] == 3)) {
+	    /* We don't support 1.5 stop bits, which is value 3. */
+	    if ((option[2] == 1) || (option[2] == 2)) {
 		val = option[2];
 		termio.c_cflag &= ~CSTOPB;
 		switch (val) {
 		case 1: break; /* 1 stop bit */
-		case 3: termio.c_cflag |= CSTOPB; break; /* 2 stop bits */
+		case 2: termio.c_cflag |= CSTOPB; break; /* 2 stop bits */
 		}
 		tcsetattr(port->devfd, TCSANOW, &termio);
 	    }
 	    if (termio.c_cflag & CSTOPB)
-		val = 3; /* 2 stop bits. */
+		val = 2; /* 2 stop bits. */
 	    else
 		val = 1; /* 1 stop bit. */
 	}
@@ -2281,13 +2296,25 @@ com_port_handler(void *cb_data, unsigned char *option, int len)
 
 	/* DTR handling */
 	case 8:
+#ifndef __CYGWIN__
 	    val = TIOCM_DTR;
 	    ioctl(port->devfd, TIOCMBIS, &val);
+#else
+	    ioctl(port->devfd, TIOCMGET, &val);
+	    val |= TIOCM_DTR;
+	    ioctl(port->devfd, TIOCMSET, &val);
+#endif
 	    goto read_dtr_val;
 
 	case 9:
+#ifndef __CYGWIN__
 	    val = TIOCM_DTR;
 	    ioctl(port->devfd, TIOCMBIC, &val);
+#else
+	    ioctl(port->devfd, TIOCMGET, &val);
+	    val &= ~TIOCM_DTR;
+	    ioctl(port->devfd, TIOCMSET, &val);
+#endif
 	    goto read_dtr_val;
 	    
 	case 7:
@@ -2302,13 +2329,25 @@ com_port_handler(void *cb_data, unsigned char *option, int len)
 
 	/* RTS handling */
 	case 11:
+#ifndef __CYGWIN__
 	    val = TIOCM_RTS;
 	    ioctl(port->devfd, TIOCMBIS, &val);
+#else
+	    ioctl(port->devfd, TIOCMGET, &val);
+	    val |= TIOCM_RTS;
+	    ioctl(port->devfd, TIOCMSET, &val);
+#endif
 	    goto read_rts_val;
 
 	case 12:
+#ifndef __CYGWIN__
 	    val = TIOCM_RTS;
 	    ioctl(port->devfd, TIOCMBIC, &val);
+#else
+	    ioctl(port->devfd, TIOCMGET, &val);
+	    val &= ~TIOCM_RTS;
+	    ioctl(port->devfd, TIOCMSET, &val);
+#endif
 	    goto read_rts_val;
 	    
 	case 10:
